=== modified file 'snapcraft/__init__.py'
--- snapcraft/__init__.py	2015-07-14 16:44:38 +0000
+++ snapcraft/__init__.py	2015-07-14 21:15:07 +0000
@@ -63,36 +63,30 @@
         except FileExistsError:
             pass
 
-    def get_source(self, source):
-        url = source
-        branchType = None
-        if url.startswith("bzr:") or url.startswith("lp:"):
-            branchType = 'bzr'
-        elif url.startswith("git:"):
-            branchType = 'git'
-        elif ':' in url:
-            raise Exception("Did not recognize branch url: " + url)
-        # Local branch
-        elif os.path.isdir(os.path.join(url, '.bzr')):
-            branchType = 'bzr'
-            url = os.path.abspath(url)
-        elif os.path.isdir(os.path.join(url, '.git')):
-            branchType = 'git'
-            url = os.path.abspath(url)
+    def get_source(self, source, source_type=None, source_tag=None, source_branch=None):
+        if source_type is None:
+            if source.startswith("bzr:") or source.startswith("lp:"):
+                source_type = 'bzr'
+            elif source.startswith("git:"):
+                source_type = 'git'
+            elif ':' in source:
+                snapcraft.common.fatal("Unrecognized source '%s' for part '%s'.  Try specifying 'source-type'." % (source, self.name))
 
-        if branchType == 'bzr':
-            if not self._pull_bzr(url):
+        if source_type == 'bzr':
+            if source_branch:
+                snapcraft.common.fatal("You can't specify source-branch for a bzr source (part '%s')." % self.name)
+            if not self._pull_bzr(source, source_tag):
                 return False
             if not self.run(['cp', '-Trfa', self.sourcedir, self.builddir]):
                 return False
-        elif branchType == "git":
-            if not self._pull_git(url):
+        elif source_type == 'git':
+            if not self._pull_git(source, source_tag, source_branch):
                 return False
             if not self.run(['cp', '-Trfa', self.sourcedir, self.builddir]):
                 return False
         else:
-            # local branch
-            path = os.path.abspath(url)
+            # local source dir
+            path = os.path.abspath(source)
             if os.path.isdir(self.builddir):
                 os.rmdir(self.builddir)
             else:
@@ -102,18 +96,35 @@
         return True
 
     # Private helpers
-    def _pull_bzr(self, url):
+    def _pull_bzr(self, source, source_tag):
+        tag_opts = []
+        if source_tag:
+            tag_opts = ['-r', 'tag:' + source_tag]
         if os.path.exists(os.path.join(self.sourcedir, ".bzr")):
-            return self.run(['bzr', 'pull', url], cwd=self.sourcedir)
+            return self.run(['bzr', 'pull'] + tag_opts + [source, '-d', self.sourcedir], cwd=os.getcwd())
         else:
             os.rmdir(self.sourcedir)
-            return self.run(['bzr', 'branch', url, self.sourcedir])
-
-    def _pull_git(self, url):
+            return self.run(['bzr', 'branch'] + tag_opts + [source, self.sourcedir], cwd=os.getcwd())
+
+    def _pull_git(self, source, source_tag, source_branch):
+        if source_tag and source_branch:
+            snapcraft.common.fatal("You can't specify both source-tag and source-branch for a git source (part '%s')." % self.name)
+
         if os.path.exists(os.path.join(self.sourcedir, ".git")):
-            return self.run(['git', 'pull'], cwd=self.sourcedir)
+            refspec = 'HEAD'
+            if source_branch:
+                refspec = 'refs/heads/' + source_branch
+            elif source_tag:
+                refspec = 'refs/tags/' + source_tag
+            return self.run(['git', '-C', self.sourcedir, 'pull', source, refspec], cwd=os.getcwd())
         else:
-            return self.run(['git', 'clone', url, '.'], cwd=self.sourcedir)
+            branch_opts = []
+            if source_tag or source_branch:
+                branch_opts = ['--branch', source_tag or source_branch]
+            return self.run(['git', 'clone'] + branch_opts + [source, self.sourcedir], cwd=os.getcwd())
 
     def _handle_source_options(self):
-        return self.get_source(self.options.source)
+        return self.get_source(self.options.source,
+                               source_type=self.options.source_type,
+                               source_tag=self.options.source_tag,
+                               source_branch=self.options.source_branch)

=== modified file 'snapcraft/common.py'
--- snapcraft/common.py	2015-07-13 21:34:39 +0000
+++ snapcraft/common.py	2015-07-14 21:15:07 +0000
@@ -18,6 +18,7 @@
 
 import os
 import subprocess
+import sys
 import tempfile
 
 env = []
@@ -41,6 +42,12 @@
 def log(msg, file=None):
     print('\033[01m' + msg + '\033[0m', file=None)
 
+
+def fatal(msg):
+    log(msg, file=sys.stderr)
+    sys.exit(1)
+
+
 commandOrder = ["pull", "build", "stage", "snap"]
 stagedir = os.path.join(os.getcwd(), "stage")
 snapdir = os.path.join(os.getcwd(), "snap")

=== modified file 'snapcraft/plugin.py'
--- snapcraft/plugin.py	2015-07-14 17:26:39 +0000
+++ snapcraft/plugin.py	2015-07-14 21:15:07 +0000
@@ -66,14 +66,18 @@
                     all_options = self.config.get('options', {})
                     if self.config.get('accepts-source-options', False):
                         all_options.setdefault('source', {'required': True})
+                        all_options.setdefault('source-type', {})
+                        all_options.setdefault('source-branch', {})
+                        all_options.setdefault('source-tag', {})
                     for opt in all_options:
+                        attrname = opt.replace('-', '_')
                         if opt in properties:
-                            setattr(options, opt, properties[opt])
+                            setattr(options, attrname, properties[opt])
                         else:
-                            if self.config['options'][opt].get('required', False):
+                            if all_options[opt].get('required', False):
                                 snapcraft.common.log("Required field %s missing on part %s" % (opt, name), file=sys.stderr)
                                 return
-                            setattr(options, opt, None)
+                            setattr(options, attrname, None)
 
                 moduleName = self.config.get('module', name)
 

=== added directory 'tests/plainbox/data/bzr-head'
=== added file 'tests/plainbox/data/bzr-head/snapcraft.yaml'
--- tests/plainbox/data/bzr-head/snapcraft.yaml	1970-01-01 00:00:00 +0000
+++ tests/plainbox/data/bzr-head/snapcraft.yaml	2015-07-14 21:15:07 +0000
@@ -0,0 +1,5 @@
+parts:
+    bzr:
+        plugin: make-project
+        source: .
+        source-type: bzr

=== added directory 'tests/plainbox/data/bzr-tag'
=== added file 'tests/plainbox/data/bzr-tag/snapcraft.yaml'
--- tests/plainbox/data/bzr-tag/snapcraft.yaml	1970-01-01 00:00:00 +0000
+++ tests/plainbox/data/bzr-tag/snapcraft.yaml	2015-07-14 21:15:07 +0000
@@ -0,0 +1,6 @@
+parts:
+    bzr:
+        plugin: make-project
+        source: .
+        source-type: bzr
+        source-tag: initial

=== added directory 'tests/plainbox/data/git-branch'
=== added file 'tests/plainbox/data/git-branch/snapcraft.yaml'
--- tests/plainbox/data/git-branch/snapcraft.yaml	1970-01-01 00:00:00 +0000
+++ tests/plainbox/data/git-branch/snapcraft.yaml	2015-07-14 21:15:07 +0000
@@ -0,0 +1,6 @@
+parts:
+    git:
+        plugin: make-project
+        source: .
+        source-type: git
+        source-branch: second

=== added directory 'tests/plainbox/data/git-head'
=== added file 'tests/plainbox/data/git-head/snapcraft.yaml'
--- tests/plainbox/data/git-head/snapcraft.yaml	1970-01-01 00:00:00 +0000
+++ tests/plainbox/data/git-head/snapcraft.yaml	2015-07-14 21:15:07 +0000
@@ -0,0 +1,5 @@
+parts:
+    git:
+        plugin: make-project
+        source: .
+        source-type: git

=== added directory 'tests/plainbox/data/git-tag'
=== added file 'tests/plainbox/data/git-tag/snapcraft.yaml'
--- tests/plainbox/data/git-tag/snapcraft.yaml	1970-01-01 00:00:00 +0000
+++ tests/plainbox/data/git-tag/snapcraft.yaml	2015-07-14 21:15:07 +0000
@@ -0,0 +1,6 @@
+parts:
+    git:
+        plugin: make-project
+        source: .
+        source-type: git
+        source-tag: initial

=== modified file 'tests/plainbox/units/jobs.pxu'
--- tests/plainbox/units/jobs.pxu	2015-07-10 19:21:16 +0000
+++ tests/plainbox/units/jobs.pxu	2015-07-14 21:15:07 +0000
@@ -89,3 +89,79 @@
     sed -i '/after/d' snapcraft.yaml
     snapcraft snap
     test $? = 1 || exit 1
+
+id: snapcraft/normal/bzr-head
+plugin: shell
+estimated_duration: 0.3
+command:
+    set -ex
+    cp -rT $PLAINBOX_PROVIDER_DATA/bzr-head .
+    bzr init .
+    bzr commit -m "1" --unchanged
+    bzr commit -m "2" --unchanged
+    snapcraft pull # test initial branch
+    test "$(bzr log -r -1 parts/bzr/src | grep ^revno:)" = "revno: 2"
+    snapcraft pull # test pull doesn't fail
+    test "$(bzr log -r -1 parts/bzr/src | grep ^revno:)" = "revno: 2"
+
+id: snapcraft/normal/bzr-tag
+plugin: shell
+estimated_duration: 0.3
+command:
+    set -ex
+    cp -rT $PLAINBOX_PROVIDER_DATA/bzr-tag .
+    bzr init .
+    bzr commit -m "1" --unchanged
+    bzr commit -m "2" --unchanged
+    bzr tag -r 1 initial
+    snapcraft pull # test initial branch
+    test "$(bzr log -r -1 parts/bzr/src | grep ^revno:)" = "revno: 1"
+    snapcraft pull # test pull doesn't fail
+    test "$(bzr log -r -1 parts/bzr/src | grep ^revno:)" = "revno: 1"
+
+id: snapcraft/normal/git-head
+plugin: shell
+estimated_duration: 0.3
+command:
+    set -ex
+    cp -rT $PLAINBOX_PROVIDER_DATA/git-head .
+    git init .
+    git commit -m "1" --allow-empty
+    git commit -m "2" --allow-empty
+    snapcraft pull
+    test "$(git -C parts/git/src log -1 --oneline | cut -d' ' -f2)" = "2"
+    snapcraft pull
+    test "$(git -C parts/git/src log -1 --oneline | cut -d' ' -f2)" = "2"
+
+id: snapcraft/normal/git-tag
+plugin: shell
+estimated_duration: 0.3
+command:
+    set -ex
+    cp -rT $PLAINBOX_PROVIDER_DATA/git-tag .
+    git init .
+    git commit -m "1" --allow-empty
+    git commit -m "2" --allow-empty
+    git tag initial HEAD@{1}
+    snapcraft pull
+    test "$(git -C parts/git/src log -1 --oneline | cut -d' ' -f2)" = "1"
+    snapcraft pull
+    test "$(git -C parts/git/src log -1 --oneline | cut -d' ' -f2)" = "1"
+
+id: snapcraft/normal/git-branch
+plugin: shell
+estimated_duration: 0.3
+command:
+    set -ex
+    cp -rT $PLAINBOX_PROVIDER_DATA/git-branch .
+    git init .
+    git commit -m "1" --allow-empty
+    git commit -m "2" --allow-empty
+    git branch second HEAD@{1}
+    git checkout second
+    git commit -m "3" --allow-empty
+    git checkout master
+    snapcraft pull
+    test "$(git -C parts/git/src log -2 --oneline | cut -d' ' -f2 | tr '\n' ' ')" = "3 1 "
+    snapcraft pull
+    test "$(git -C parts/git/src log -2 --oneline | cut -d' ' -f2 | tr '\n' ' ')" = "3 1 "

